@@ -3,7 +3,7 @@
 
  A signal handler which cleans up an application and exits.
 
- Provided under GPL license within cdrskin and under BSD license elsewise.
+ Provided under GPL license within GPL projects, BSD license elsewise.
 */
 
 #ifndef Cleanup_includeD
@@ -13,7 +13,7 @@
 /** Layout of an application provided cleanup function using an application
     provided handle as first argument and the signal number as second
     argument. The third argument is a flag bit field with no defined bits yet.
-    If the handler returns 2 then it has delegated exit() to some other
+    If the handler returns 2 or -2 then it has delegated exit() to some other
     instance and the Cleanup handler shall return rather than exit.
 */
 typedef int (*Cleanup_app_handler_T)(void *, int, int);
