@@ -353,7 +353,7 @@ void burn_disc_erase(struct burn_drive *drive, int fast)
 	/* a ssert(!SCAN_GOING()); */
 	/* a ssert(!find_worker(drive)); */
 
-	if((drive == NULL)) {
+	if(drive == NULL) {
 		libdax_msgs_submit(libdax_messenger, -1,
 			0x00020104,
 			LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
