@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2012 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2013 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -537,6 +537,10 @@ int burn_drive_grab(struct burn_drive *d, int le)
 		{ret = 0; goto ex;}
 
 ex:;
+	if (d->cancel || burn_is_aborting(0)) {
+	        d->unlock(d);
+		d->release(d);
+	}
 	d->silent_on_scsi_error = sose;
 	d->busy = BURN_DRIVE_IDLE;
 	burn_grab_restore_sig_action(signal_action_mem, 0);
@@ -570,6 +574,7 @@ struct burn_drive *burn_drive_register(struct burn_drive *d)
 	d->toc_entry = NULL;
 	d->disc = NULL;
 	d->erasable = 0;
+	d->write_opts = NULL;
 
 #ifdef Libburn_ticket_62_re_register_is_possiblE
 	/* ts A60904 : ticket 62, contribution by elmom */
@@ -675,6 +680,10 @@ int burn_drive_mark_unready(struct burn_drive *d, int flag)
 		free(d->toc_entry);
 	d->toc_entry = NULL;
 	d->toc_entries = 0;
+	if (d->write_opts != NULL) {
+		burn_write_opts_free(d->write_opts);
+		d->write_opts = NULL;
+	}
 	if (d->disc != NULL) {
 		burn_disc_free(d->disc);
 		d->disc = NULL;
@@ -844,7 +853,7 @@ void burn_wait_all(void)
 
 void burn_disc_erase_sync(struct burn_drive *d, int fast)
 {
-	int ret;
+	int ret, was_error = 0;
 
 	if (d->drive_role == 5) { /* Random access write-only drive */
 		ret = truncate(d->devname, (off_t) 0);
@@ -898,12 +907,16 @@ void burn_disc_erase_sync(struct burn_drive *d, int fast)
 		ret = d->get_erase_progress(d);
 		if (ret == -2 || ret > 0)
 	break;
+		if (ret == -3)
+			was_error = 1;
 		sleep(1);
 	}
 	while (1) {
 		ret = d->get_erase_progress(d);
 		if(ret == -2)
 	break;
+		if (ret == -3)
+			was_error = 1;
 		if (ret >= 0)
 			d->progress.sector = ret;
 		sleep(1);
@@ -918,6 +931,8 @@ void burn_disc_erase_sync(struct burn_drive *d, int fast)
 	if (d->drive_role == 1)
 		burn_drive_inquire_media(d);
 	d->busy = BURN_DRIVE_IDLE;
+	if (was_error)
+		d->cancel = 1;
 }
 
 /*
@@ -927,6 +942,7 @@ void burn_disc_erase_sync(struct burn_drive *d, int fast)
 void burn_disc_format_sync(struct burn_drive *d, off_t size, int flag)
 {
 	int ret, buf_secs, err, i, stages = 1, pbase, pfill, pseudo_sector;
+	int was_error = 0;
 	off_t num_bufs;
 	char msg[80];
 	struct buffer *buf = NULL, *buf_mem = d->buffer;
@@ -971,12 +987,16 @@ void burn_disc_format_sync(struct burn_drive *d, off_t size, int flag)
 		ret = d->get_erase_progress(d);
 		if (ret == -2 || ret > 0)
 	break;
+		if (ret == -3)
+			was_error = 1;
 		sleep(1);
 	}
 	while (1) {
 		pseudo_sector = d->get_erase_progress(d);
 		if(pseudo_sector == -2)
 	break;
+		if (pseudo_sector == -3)
+			was_error = 1;
 		if (pseudo_sector >= 0)
 			d->progress.sector = pseudo_sector / stages;
 		sleep(1);
@@ -1040,6 +1060,8 @@ ex:;
 	d->progress.sector = 0x10000;
 	d->busy = BURN_DRIVE_IDLE;
 	d->buffer = buf_mem;
+	if (was_error)
+		d->cancel = 1;
 	BURN_FREE_MEM(buf);
 }
 
@@ -1852,8 +1874,10 @@ int burn_drive_scan_and_grab(struct burn_drive_info *drive_infos[], char* adr,
 */
 
 	ret = burn_drive_grab(drive_infos[0]->drive, load);
-	if (ret != 1)
+	if (ret != 1) {
+		burn_drive_forget(drive_infos[0]->drive, 0);
 		return -1;
+	}
 	return 1;
 }
 
@@ -2123,7 +2147,7 @@ int burn_drive_convert_scsi_adr(int bus_no, int host_no, int channel_no,
 	ret = 0;
 ex:;
 	if (first == 0)
-		sg_give_next_adr(&enm, fname, sizeof(fname), -1);
+		sg_give_next_adr(&enm, fname, fname_size, -1);
 	BURN_FREE_MEM(fname);
 	BURN_FREE_MEM(msg);
 	return ret;
@@ -2476,6 +2500,13 @@ int burn_disc_pretend_full(struct burn_drive *d)
 	return 1;
 }
 
+/* ts B31110 API function */
+int burn_disc_pretend_full_uncond(struct burn_drive *d)
+{
+	d->status = BURN_DISC_FULL;
+	return 1;
+}
+
 /* ts A61021: new API function */
 int burn_disc_read_atip(struct burn_drive *d)
 {
@@ -3389,3 +3420,13 @@ int burn_disc_get_leadin_text(struct burn_drive *d,
 	return ret;
 }
 
+
+/* ts B31023 API */
+/* Inquire for DVD-RW failure of TAO
+*/
+int burn_drive_was_feat21_failure(struct burn_drive *d)
+{
+	return !!d->was_feat21h_failure;
+}
+
+
