@@ -1,10 +1,14 @@
 
 /* libdax_msgs
    Message handling facility of libdax.
-   Copyright (C) 2006 - 2009 Thomas Schmitt <scdbackup@gmx.net>,
-   provided under GPL version 2
+   Copyright (C) 2006 - 2010 Thomas Schmitt <scdbackup@gmx.net>,
+   provided under GPL version 2 or later.
 */
 
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
 #include <stdio.h>
 #include <sys/types.h>
 #include <unistd.h>
@@ -12,6 +16,7 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <sys/time.h>
+#include <pthread.h>
 
 /* Only this single source module is entitled to do this */ 
 #define LIBDAX_MSGS_H_INTERNAL 1
@@ -32,7 +37,7 @@ static int libdax_msgs_item_new(struct libdax_msgs_item **item,
  struct timezone tz;
 
  (*item)= o= 
-           (struct libdax_msgs_item *) malloc(sizeof(struct libdax_msgs_item));
+        (struct libdax_msgs_item *) calloc(1, sizeof(struct libdax_msgs_item));
  if(o==NULL)
    return(-1);
  o->timestamp= 0.0;
@@ -135,7 +140,7 @@ int libdax_msgs_new(struct libdax_msgs **m, int flag)
 {
  struct libdax_msgs *o;
 
- (*m)= o= (struct libdax_msgs *) malloc(sizeof(struct libdax_msgs));
+ (*m)= o= (struct libdax_msgs *) calloc(1, sizeof(struct libdax_msgs));
  if(o==NULL)
    return(-1);
  o->refcount= 1;
@@ -284,7 +289,7 @@ int libdax_msgs__sev_to_text(int severity, char **severity_name,
                              int flag)
 {
  if(flag&1) {
-   *severity_name= "NEVER\nABORT\nFATAL\nFAILURE\nMISHAP\nSORRY\nWARNING\nHINT\nNOTE\nUPDATE\nDEBUG\nERRFILE\nALL";
+   *severity_name= "ALL ERRFILE DEBUG UPDATE NOTE HINT WARNING SORRY MISHAP FAILURE FATAL ABORT NEVER";
    return(1);
  }
  *severity_name= "";
@@ -322,6 +327,11 @@ int libdax_msgs__sev_to_text(int severity, char **severity_name,
 }
 
 
+/*
+    @param flag Bitfield for control purposes
+                    bit0= If direct output to stderr:
+                          CarriageReturn rather than LineFeed
+*/
 int libdax_msgs_submit(struct libdax_msgs *m, int origin, int error_code,
                        int severity, int priority, char *msg_text,
                        int os_errno, int flag)
@@ -340,7 +350,8 @@ int libdax_msgs_submit(struct libdax_msgs *m, int origin, int error_code,
    if(ret>0)
      sprintf(sev_text,"%s : ",sev_name);
 
-   fprintf(stderr,"%s%s%s\n",m->print_id,sev_text,textpt);
+   fprintf(stderr, "%s%s%s%c", m->print_id, sev_text, textpt,
+                               (flag & 1) ? '\r' : '\n');
    if(os_errno!=0) {
      ret= libdax_msgs_lock(m,0);
      if(ret<=0)
@@ -365,7 +376,7 @@ int libdax_msgs_submit(struct libdax_msgs *m, int origin, int error_code,
  item->severity= severity;
  item->priority= priority;
  if(msg_text!=NULL) {
-   item->msg_text= malloc(strlen(msg_text)+1);
+   item->msg_text= calloc(1, strlen(msg_text)+1);
    if(item->msg_text==NULL)
      goto failed;
    strcpy(item->msg_text,msg_text);
