@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* 
-   Copyright (c) 2006 - 2011 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2013 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later
         and under FreeBSD license revised, i.e. without advertising clause.
 */
@@ -448,7 +448,7 @@ static void enumerate_common(char *fname, int bus_no, int host_no,
 	out.channel = channel_no;
 	out.lun = lun_no;
 
-	out.devname = burn_strdup(fname);
+	out.devname = strdup(fname);
 
 	out.cam = NULL;
 	out.lock_fd = -1;
@@ -477,7 +477,7 @@ static void enumerate_common(char *fname, int bus_no, int host_no,
 	out.read_toc = mmc_read_toc;
 	out.write = mmc_write;
 	out.erase = mmc_erase;
-	out.read_sectors = mmc_read_sectors;
+	out.read_cd = mmc_read_cd;
 	out.perform_opc = mmc_perform_opc;
 	out.set_speed = mmc_set_speed;
 	out.send_parameters = spc_select_error_params;
@@ -768,10 +768,11 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 
 	mmc_function_spy(NULL, "sg_issue_command");
 
-	if (d->cam == NULL) {
-		c->error = 0;
+	c->error = 0;
+	memset(c->sense, 0, sizeof(c->sense));
+
+	if (d->cam == NULL)
 		return 0;
-	}
 	if (burn_sg_log_scsi & 1) {
 		if (fp == NULL) {
 			fp= fopen("/tmp/libburn_sg_command_log", "a");
@@ -783,6 +784,10 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 		scsi_log_cmd(c,fp,0);
 
 	c->error = 0;
+	if (c->timeout > 0)
+		timeout_ms = c->timeout;
+	else
+		timeout_ms = 200000;
 
 	ccb = cam_getccb(d->cam);
 	cam_fill_csio(&ccb->csio,
@@ -794,7 +799,7 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 				  0,                              /* dxfer_len */
 				  sizeof (ccb->csio.sense_data),  /* sense_len */
 				  0,                              /* cdb_len */
-				  30*1000);                       /* timeout */
+				  timeout_ms);                    /* timeout */
 	switch (c->dir) {
 	case TO_DRIVE:
 		ccb->csio.ccb_h.flags |= CAM_DIR_OUT;
@@ -827,8 +832,6 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 	ccb->csio.cdb_len = c->oplen;
 	memcpy(&ccb->csio.cdb_io.cdb_bytes, &c->opcode, c->oplen);
 	
-	memset(&ccb->csio.sense_data, 0, sizeof (ccb->csio.sense_data));
-
 	if (c->page) {
 		ccb->csio.data_ptr  = c->page->data;
 		if (c->dir == FROM_DRIVE) {
@@ -854,11 +857,15 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 	}
 
 	start_time = time(NULL);
-	timeout_ms = 200000;
 	for (i = 0; !done; i++) {
+
+		memset(&ccb->csio.sense_data, 0, sizeof(ccb->csio.sense_data));
 		memset(c->sense, 0, sizeof(c->sense));
+		c->start_time = burn_get_time(0);
+
 		err = cam_send_ccb(d->cam, ccb);
 
+		c->end_time = burn_get_time(0);
 		ignore_error = sense_len = 0;
 		/* ts B00325 : CAM_AUTOSNS_VALID advised by Alexander Motin */
 		if (ccb->ccb_h.status & CAM_AUTOSNS_VALID) {
@@ -955,17 +962,12 @@ int sg_issue_command(struct burn_drive *d, struct command *c)
 				c->sense[13] = 0x00;
 				done = 1;
 			}
-
-			/* >>> Need own duration time measurement.
-			       Then remove bit1 from flag.
-			*/
-			done = scsi_eval_cmd_outcome(d, c, fp, c->sense,
-						sense_len, 0, start_time,
-						timeout_ms, i,
-						2 | !!ignore_error);
-		} else {
-			done = 1;
 		}
+		done = scsi_eval_cmd_outcome(d, c, fp, c->sense,
+						sense_len, start_time,
+						timeout_ms, i, !!ignore_error);
+		if (d->cancel)
+			done = 1;
 	} while (!done);
 	ret = 1;
 ex:;
