@@ -76,11 +76,15 @@ extern struct libdax_msgs *libdax_messenger;
 
 /* The size to be used with BD-RE media in normal, not streamed mode.
 */
-#define Libburn_bd_re_obS (32 * 1024)
+#define Libburn_bd_re_obS (64 * 1024)
 
-/* The size to be used with BD-RE media in streamed mode.
+/* The size to be used with BD-R media in normal, not streamed mode.
 */
-#define Libburn_bd_re_streamed_obS (64 * 1024)
+#define Libburn_bd_r_obS (64 * 1024)
+
+/* The size to be used with BD-RE and BD-R media in streamed mode.
+*/
+#define Libburn_bd_streamed_obS (64 * 1024)
 
 
 static int type_to_ctrl(int mode)
@@ -455,7 +459,7 @@ struct cue_sheet *burn_create_toc_entries(struct burn_write_opts *o,
 					  struct burn_session *session,
 					  int nwa)
 {
-	int i, m, s, f, form, pform, runtime = -150, ret, track_length;
+	int i, m, s, f, form, runtime = -150, ret, track_length;
 	int leadin_form, leadin_start, pregap = 150, postgap;
 	unsigned char ctladr, scms;
 	struct burn_drive *d;
@@ -469,6 +473,8 @@ struct cue_sheet *burn_create_toc_entries(struct burn_write_opts *o,
 
 #ifdef Libburn_track_multi_indeX
 	int j;
+#else
+	int pform;
 #endif
 
 	if (ntr < 1) {
@@ -506,11 +512,13 @@ struct cue_sheet *burn_create_toc_entries(struct burn_write_opts *o,
 			"Track mode has unusable value", 0, 0);
 		goto failed;
 	}
-	if (o->num_text_packs > 0) {
-		leadin_form = 0x41;
-	} else {
+	if (tar[0]->mode & BURN_AUDIO)
 		leadin_form = 0x01;
-
+	else
+		leadin_form = 0x14;
+	if (o->num_text_packs > 0) {
+		leadin_form |= 0x40;
+	} else {
 		/* Check for CD-TEXT in session. Not the final creation,
 		   because the cue sheet content might be needed for CD-TEXT
 		   pack type 0x88 "TOC".
@@ -520,7 +528,7 @@ struct cue_sheet *burn_create_toc_entries(struct burn_write_opts *o,
 			if (ret < 0)
 				goto failed;
 			else if (ret > 0)
-				leadin_form = 0x41;
+				leadin_form |= 0x40;
 		}
 	}
 
@@ -600,7 +608,10 @@ struct cue_sheet *burn_create_toc_entries(struct burn_write_opts *o,
 	if (tar[0]->pregap2_size < 150)
 		tar[0]->pregap2_size = 150;
 
+#ifndef Libburn_track_multi_indeX
 	pform = form;
+#endif
+
 	for (i = 0; i < ntr; i++) {
 
 		/* ts A70125 : 
@@ -788,13 +799,19 @@ XXX this is untested :)
 			rem -= burn_sector_length(tar[i]->mode);
 			runtime--;
 		}
+
+#ifndef Libburn_track_multi_indeX
 		pform = form;
+#endif
+
 	}
 	burn_lba_to_msf(runtime, &m, &s, &f);
 	e[2].pmin = m;
 	e[2].psec = s;
 	e[2].pframe = f;
-	ret = add_cue(sheet, ctladr | 1, 0xAA, 1, 1, 0, runtime);
+
+	ret = add_cue(sheet, ctladr | 1, 0xAA, 1, leadin_form & 0x3f,
+                      0, runtime);
 	if (ret <= 0)
 		goto failed;
 	return sheet;
@@ -1301,7 +1318,7 @@ int burn_disc_init_write_status(struct burn_write_opts *o,
 {
 	struct burn_drive *d = o->drive;
 	struct burn_track *t = NULL;
-	int sx, tx;
+	int sx, tx, ret;
 
 	d->cancel = 0;
 
@@ -1346,6 +1363,13 @@ int burn_disc_init_write_status(struct burn_write_opts *o,
 	if (o->fill_up_media && t != NULL)
 		burn_track_set_fillup(t, 1);
 
+	d->was_feat21h_failure = 0;
+	if(d->write_opts != NULL)
+		burn_write_opts_free(d->write_opts);
+	ret = burn_write_opts_clone(o, &(d->write_opts), 0);
+	if (ret <= 0)
+		return ret;
+
 	d->busy = BURN_DRIVE_WRITING;
 
 	return 1;
@@ -1588,15 +1612,29 @@ int burn_disc_open_track_dvd_plus_r(struct burn_write_opts *o,
 
 	if (o->write_type == BURN_WRITE_SAO &&
 	    ! burn_track_is_open_ended(s->track[tnum])) {
- 		/* Round track size up to write chunk size and reserve track */
+ 		/* Reserve track */
 		size = ((off_t) burn_track_get_sectors_2(s->track[tnum], 1))
 			* (off_t) 2048;
-		/* o->obs should be 32k or 64k already. But 32k alignment
-		  was once performed in d->reserve_track() */
-		if (o->obs % 32768)
-			o->obs += 32768 - (o->obs % 32768);
-		if (size % o->obs)
-			size += (off_t) (o->obs - (size % o->obs));
+		if (o->obs_pad) {
+	 		/* Round track size up to write chunk size */
+			/* o->obs should be 32k or 64k already. But 32k
+			   alignment was once performed in d->reserve_track()*/
+			if (o->obs % 32768)
+				o->obs += 32768 - (o->obs % 32768);
+			if (size % o->obs)
+				size += (off_t) (o->obs - (size % o->obs));
+		}
+
+		/* <<< Only for now until the first DVD+R succeeded */
+		if (!o->obs_pad) {
+			sprintf(msg, "Program error: encountered DVD+R without chunk padding");
+			libdax_msgs_submit(libdax_messenger, d->global_index,
+				0x00000004,
+				LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
+				msg, 0, 0);
+			{ret = 0; goto ex;}
+		}
+
 		ret = d->reserve_track(d, size);
 		if (ret <= 0) {
 			sprintf(msg, "Cannot reserve track of %.f bytes",
@@ -1834,7 +1872,7 @@ int burn_dvd_write_track(struct burn_write_opts *o,
 	struct burn_drive *d = o->drive;
 	struct buffer *out = d->buffer;
 	int sectors;
-	int i, open_ended = 0, ret= 0, is_flushed = 0;
+	int i, open_ended = 0, ret= 0, is_flushed = 0, track_open = 0;
 	int first_buf_cap = 0, further_cap = 0, buf_cap_step = 1024;
 
 	/* ts A70213 : eventually expand size of track to max */
@@ -1862,6 +1900,7 @@ int burn_dvd_write_track(struct burn_write_opts *o,
 		if (ret <= 0)
 			goto ex;
 	}
+	track_open = 1;
 
 	sectors = burn_track_get_sectors_2(t, 1);
 	open_ended = burn_track_is_open_ended(t);
@@ -1944,7 +1983,7 @@ int burn_dvd_write_track(struct burn_write_opts *o,
 ex:;
 	if (d->cancel)
 		burn_source_cancel(t->source);
-	if (!is_flushed)
+	if (track_open && !is_flushed)
 		d->sync_cache(d); /* burn_write_flush() was not called */
 	return ret;
 }
@@ -2092,13 +2131,16 @@ int burn_dvd_write_session(struct burn_write_opts *o,
 	if (d->current_profile == 0x11 || d->current_profile == 0x14 ||
 	    d->current_profile == 0x15) {
 		/* DVD-R , DVD-RW Sequential, DVD-R/DL Sequential */
-		multi_mem = o->multi;
-		if (!is_last_session)
-			o->multi = 1;
-		ret = burn_disc_close_session_dvd_minus_r(o);
-		o->multi = multi_mem;
-		if (ret <= 0)
-			return 0;
+		/* If feature 21h failed on write 0: do not close session */
+		if (d->was_feat21h_failure != 2) {
+			multi_mem = o->multi;
+			if (!is_last_session)
+				o->multi = 1;
+			ret = burn_disc_close_session_dvd_minus_r(o);
+			o->multi = multi_mem;
+			if (ret <= 0)
+				return 0;
+		}
 	} else if (d->current_profile == 0x12 || d->current_profile == 0x43) {
 		/* DVD-RAM , BD-RE */
 		/* ??? any finalization needed ? */;
@@ -2271,13 +2313,15 @@ int burn_dvd_write_sync(struct burn_write_opts *o,
 				LIBDAX_MSGS_SEV_NOTE, LIBDAX_MSGS_PRIO_HIGH,
 				msg, 0, 0);
 		}
-		o->obs_pad = 0; /* no filling-up of track's last 32k buffer */
+		if (o->obs_pad < 2)
+			o->obs_pad = 0; /* no filling-up of last 32k buffer */
 		if (d->current_profile == 0x43) /* BD-RE */
 			o->obs = Libburn_bd_re_obS;
 		if (d->do_stream_recording) {
-			o->obs_pad = 1;
+			if (o->obs_pad < 2)
+				o->obs_pad = 1;
 			if (d->current_profile == 0x43) /* BD-RE */
-				o->obs = Libburn_bd_re_streamed_obS;
+				o->obs = Libburn_bd_streamed_obS;
 		}
 
 	} else if (d->current_profile == 0x13) {
@@ -2312,7 +2356,8 @@ int burn_dvd_write_sync(struct burn_write_opts *o,
 			burn_track_set_size(t, default_size);
 		}
 		/* Whether to fill-up last 32k buffer of track. */
-		o->obs_pad = (o->write_type != BURN_WRITE_SAO);
+		if (o->obs_pad < 2)
+			o->obs_pad = (o->write_type != BURN_WRITE_SAO);
 		ret = burn_disc_setup_dvd_minus_r(o, disc);
 		if (ret <= 0) {
 			sprintf(msg,
@@ -2353,10 +2398,13 @@ int burn_dvd_write_sync(struct burn_write_opts *o,
 			goto early_failure;
 		}
 		/* ??? padding needed ??? cowardly doing it for now */
-		o->obs_pad = 1; /* fill-up track's last 32k buffer */
+		if (o->obs_pad < 2)
+			o->obs_pad = 1; /* fill-up track's last obs buffer */
+		if (d->current_profile == 0x41) /* BD-R */
+			o->obs = Libburn_bd_r_obS;
 		if (d->do_stream_recording) {
 			if (d->current_profile == 0x41) /* BD-R */
-				o->obs = Libburn_bd_re_streamed_obS;
+				o->obs = Libburn_bd_streamed_obS;
 		}
 	}
 
@@ -2386,10 +2434,10 @@ int burn_dvd_write_sync(struct burn_write_opts *o,
 
 	if (d->do_stream_recording &&
 		(d->current_profile == 0x43 || d->current_profile == 0x41) &&
-		o->obs < Libburn_bd_re_streamed_obS) {
+		o->obs < Libburn_bd_streamed_obS) {
 		/* LG GGW-H20 writes junk with stream recording and obs=32k */
 		sprintf(msg,
-		    "Stream recording disabled because of small OS buffer");
+		   "Stream recording disabled because of small output buffer");
 		libdax_msgs_submit(libdax_messenger, d->global_index,
 			 0x00020176, LIBDAX_MSGS_SEV_NOTE,
 			 LIBDAX_MSGS_PRIO_HIGH, msg, 0, 0);
@@ -2768,7 +2816,8 @@ int burn_stdio_write_sync(struct burn_write_opts *o,
 	struct burn_drive *d = o->drive;
 
 	d->needs_close_session = 0;
-	o->obs_pad = 0; /* no filling-up of track's last 32k buffer */
+	if (o->obs_pad < 2)
+		o->obs_pad = 0; /* no filling-up of track's last 32k buffer */
 	o->obs = 32*1024; /* buffer size */
 
 	if (disc->sessions != 1)
@@ -3090,6 +3139,10 @@ ex:;
 		burn_os_free_buffer((char *) d->buffer,
 					sizeof(struct buffer), 0);
 	d->buffer = buffer_mem;
+	if (d->write_opts != NULL) {
+		burn_write_opts_free(d->write_opts);
+		d->write_opts = NULL;
+	}
 	return;
 }
 
@@ -3175,6 +3228,7 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 		if (fd == -1)
 			{ret = 0; goto ex;}
 	}
+	d->cancel = 0;
 	d->busy = BURN_DRIVE_WRITING_SYNC;
 	d->buffer = buf;
 
